/** \file MiniscopeDevice.h
	\brief Interface description for miniscope project device (ADC, preamplifier...) plugins
	\author Tomasz Ostrowski, http://tomeko.net
	\date October 2008
*/

#ifndef MiniscopeDeviceH
#define MiniscopeDeviceH
//---------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>
#include "MiniscopeDevice.h"

// export/import declarations switch
// define _EXPORTING when (before) including file in dll

#ifdef _EXPORTING
#define DECLARE_FN(type, fn, ...) __declspec(dllexport) type __stdcall fn(__VA_ARGS__)
#else
#define DECLARE_FN(type, fn, ...) __declspec(dllimport) type __stdcall fn(__VA_ARGS__); \
	typedef type __stdcall (*pf##fn)(__VA_ARGS__)
#endif

#ifdef _EXPORTING
	#define DECLSPEC __declspec(dllexport)
#else
	#define DECLSPEC __declspec(dllimport)
#endif

enum { DLL_INTERFACE_MAJOR_VERSION = 0 };
enum { DLL_INTERFACE_MINOR_VERSION = 1 };

struct S_DEVICE_DLL_INTERFACE
{
	int majorVersion;
	int minorVersion;
};

enum E_CONNECTION_STATE
{
	DEVICE_DISCONNECTED = 0,
	DEVICE_CONNECTING,
	DEVICE_CONNECTED,
	DEVICE_DISCONNECTING
};

struct S_SCOPE
{
    int iId;
    char szDescription;
};

struct S_SCOPE_CAPABILITIES;

// callback declarations
typedef void __stdcall (*CALLBACK_LOG)(void *cookie, char *szText);                   ///< called to generate log in parent application
typedef void __stdcall (*CALLBACK_CONNECT)(void *cookie, int state, char *szMsgText); ///< called on connection/disconnection of device
typedef void __stdcall (*CALLBACK_DATA)(void *cookie, void *buffer, unsigned int buffersize); ///< called to pass data (samples) to parent application
typedef void __stdcall (*CALLBACK_TRIGGER)(void *cookie);

// exported/imported function set

DECLARE_FN(void, GetMiniscopeInterfaceDescription, struct S_DEVICE_DLL_INTERFACE*);

DECLARE_FN(void, SetCallbacks, void *cookie, CALLBACK_LOG lpLog, CALLBACK_CONNECT lpConnect, CALLBACK_DATA lpData, CALLBACK_TRIGGER lpTrigger);

DECLARE_FN(void, GetDeviceCapabilities, struct S_SCOPE_CAPABILITIES **caps);

DECLARE_FN(void, ShowSettings, HANDLE parent);

DECLARE_FN(int, SetSensitivity, int iChannel, int iId);
DECLARE_FN(int, SetSamplingPeriod, int iId);
DECLARE_FN(int, SetTriggerSource, int iChannel, int iId);
DECLARE_FN(int, SetCouplingType, int iChannel, int iId);
DECLARE_FN(int, SetBufferSize, int iId);
DECLARE_FN(int, SetOffset, int iChannel, float fOffset);

DECLARE_FN(int, Connect, void);
DECLARE_FN(int, Disconnect, void);

DECLARE_FN(int, AcquireStart, void);
DECLARE_FN(int, AcquireStop, void);
DECLARE_FN(int, ManualTrigger, void);



#ifdef __cplusplus
}
#endif

#endif
